;--------------------------------------------------------------------------
;
;  Filename:    lpNonStreaming.asm   NON-STREAMING version
;
;  Description: User functions to access the LP radio.
;               Contains functions and support for to actually perform 
;               transmit and receive operations.
;
;               "Non-streaming" means the entire packet fits in the 16-byte
;                buffer in the radio without overflowing.
;
;               "Streaming" means the buffer must be monitored and serviced
;                while the Tx or Rx transaction is in progress because the 
;                16-byte hardware buffer can't hold the entire packet.
;
;--------------------------------------------------------------------------
; WirelessUSB LP Radio Driver Version 1.1
;--------------------------------------------------------------------------
;
; Copyright 2005-2006, Cypress Semiconductor Corporation.
;
; This software is owned by Cypress Semiconductor Corporation (Cypress)
; and is protected by and subject to worldwide patent protection (United
; States and foreign), United States copyright laws and international
; treaty provisions. Cypress hereby grants to licensee a personal,
; non-exclusive, non-transferable license to copy, use, modify, create
; derivative works of, and compile the Cypress Source Code and derivative
; works for the sole purpose of creating custom software in support of
; licensee product to be used only in conjunction with a Cypress integrated
; circuit as specified in the applicable agreement. Any reproduction,
; modification, translation, compilation, or representation of this
; software except as specified above is prohibited without the express
; written permission of Cypress.
;
; Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
; WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
; Cypress reserves the right to make changes without further notice to the
; materials described herein. Cypress does not assume any liability arising
; out of the application or use of any product or circuit described herein.
; Cypress does not authorize its products for use as critical components in
; life-support systems where a malfunction or failure may reasonably be
; expected to result in significant injury to the user. The inclusion of
; Cypress' product in a life-support systems application implies that the
; manufacturer assumes all risk of such use and in doing so indemnifies
; Cypress against all charges.
;
; Use may be limited by and subject to the applicable Cypress software
; license agreement.
;
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------;
;                                                                          ;
;                      I N C L U D E   F I L E S                           ;
;                                                                          ;
;--------------------------------------------------------------------------;

                   INCLUDE "lpradio.inc"
                   INCLUDE "spim_radio.inc"
                   INCLUDE "psocgpioint.inc"
                   INCLUDE "lpregs.inc"
                   INCLUDE "m8c.inc"
                   INCLUDE "lpIrqMacros.inc"
                   INCLUDE "memory.inc"

;--------------------------------------------------------------------------;
;                                                                          ;
;                  M A C R O   D E F I N I T I O N S                       ;
;                                                                          ;
;--------------------------------------------------------------------------;

macro ISR_REG_PRESERVE (IOReg)
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        REG_PRESERVE (@IOReg)
  ENDIF
endm

macro ISR_REG_RESTORE (IOReg)
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        REG_RESTORE (@IOReg)
  ENDIF
endm

macro TST_IRQ_PIN
        TST     REG[LP_IRQ_Data_ADDR], LP_IRQ_MASK
endm

LP_FIFO_SIZE:   EQU     16      ; Radio Rx/Tx FIFO size
LP_FIFO_HALF:   EQU     8       ; Radio Rx/Tx FIFO half-way mark (B8 flag)

;--------------------------------------------------------------------------;
;                                                                          ;
;                            V A R I A B L E S                             ;
;                                                                          ;
;--------------------------------------------------------------------------;

                   AREA    InterruptRAM(ram)

RadioBytesRead::   BLK  1
RadioScratch0:     BLK  1


;--------------------------------------------------------------------------;
;                                                                          ;
;                                 C O D E                                  ;
;                                                                          ;
;--------------------------------------------------------------------------;

                   AREA    UserModules (ROM, REL)
                   DISABLE_CODE_COMPRESSION

;--------------------------------------------------------------------------;
;                                                                          ;
;                 T R A N S M I T                                          ;
;                                                                          ;
;--------------------------------------------------------------------------;
.section
;-----------------------------------------------------------------------------
;
; RadioStartTransmit:
;                  Start the transmission of a packet. The location of the
;                  packet buffer to transmit must have previously been set
;                  by calling RadioSetPtr().
;
;                  After starting the transmission of a packet with this call,
;                  the transmit operation should be monitored via
;                  RadioGetTransmitState(). When RadioGetTransmitState()
;                  indicates that the transmission has completed, then
;                  RadioEndTransmit() should be called.
;
;                  After calling RadioStartTransmit() NO CALLS can be made to
;                  configuration access routines until transmit operation is
;                  terminated by calling RadioEndTransmit() or RadioAbort().
;                  Until a call is made to end the transmit operation
;                  the only other call supported is RadioGetTransmitState(). 
;
; 'C' Call:        void RadioStartTransmit(BYTE retryCount, RADIO_LENGTH len);
;                  (A call to RadioSetPtr must have been made prior to the
;                  call to RadioStartTransmit.)
;
; Assembly Call:   A: retry count
;                  X: packet length
;
; Assembly Return: A: State
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioStartTransmit::
 RadioStartTransmit::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioRetryCount], A    ; Save retry count
        MOV     [RadioPacketLength], X  ;  and length.

;-----------------------------------------------------------------------------
; RadioRestartTransmit - Retry because no AutoAck from prior Tx
;-----------------------------------------------------------------------------
RadioRestartTransmit:

        CALL    wakeupRadio

        MOV     [RadioState], RADIO_TX
        MOV     [RadioTemp2],(TX_GO | TX_CLR | TXC_IRQ | TXE_IRQ)

        ; --------------------------------------------------------------------
        ; TX_GO is issued (starting the radio transmitting).  
        ; IMMEDIATELY after this (during radio "start-up"), the data
        ;  is copied to the radio.  Global Interrupts are disabled, because 
        ;  the data must be in the radio before the "startup" is finished.
        ; --------------------------------------------------------------------
        PUSH_F_VIA_RAM                  ; PUSH F and disable GIE

  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSByte of local buf adr
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; Burst write the 2-byte tmp buf
        MOV     A, TX_LENGTH_ADR             ;  to start the TX.
        MOV     X, 2
        MOV     [RadioWipLen], X
        CALL    RadioBurstWriteWip      ; Write length and TX control as burst.

        MOV     X, [RadioPacketLength]
        MOV     A, TX_BUFFER_ADR
        CALL    RadioFileWrite          ; Burst the data in

        MOV     A, [RadioState]
        POP_F_RET                       ; Ret/restore GIE state
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioGetTransmitState:
;                  Returns the state of the current transmit operation.
;                  This call should be made after starting a transmit
;                  operation with the RadioStartTransmit function.
;
;                  Although bits in the status register in the hardware clear
;                  automatically, we make them sticky until RadioEndReceive.
;
; 'C' Call:        RADIO_STATE RadioGetTransmitState(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: State
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioGetTransmitState::
 RadioGetTransmitState::
        RAM_SETPAGE_CUR >RadioDriverRamPage

        TST_IRQ_PIN                     ; Has interrupt asserted?
        JZ      RGTSDone                ;  No, just return the state.
        ;
        ; IRQ indicates TX_GO is finished
        ;
RadioGetTransmitStateIsr:
        MOV     A, TX_IRQ_STATUS_ADR
        CALL    RadioReadStatusDebounced
        AND     A, (TXE_IRQ | TXC_IRQ)
        OR      [RadioState], A
        CMP     A, TXC_IRQ
        JZ      RGTSDone                ; Tx Complete without Error
                                        ; Tx Complete with    Error
; During Transaction Ack Rx window, if SOP is received
;  then get RXC/RXE (not TXC/TXE). RadioInit() ties RXC/RXE to IRQ.
;  Reading RX_IRQ_STATUS_ADR here ensures IRQ deasserts.
;
        MOV     A, RX_IRQ_STATUS_ADR    ; Clear the RX IRQ.
        CALL    RadioRead                   
        OR      [RadioState], (TXE_IRQ | TXC_IRQ)  ; Defined as Tx Error
        ;
        ; Maybe retry the failed transmit.
        ;
        DEC     [RadioRetryCount]       ; Dec and test Tx Retry count
        JNC     RadioRestartTransmit    ; Try again.

RGTSDone:
        MOV     A, [RadioState]
        RET
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioEndTransmit: Completes a transmit operation.
;
; 'C' Call:        void RadioEndTransmit(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: Preserved
;                  X: Undefined
;
_RadioEndTransmit::
 RadioEndTransmit::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioState], RADIO_IDLE        ; Clear our status.
        RET
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioBlockingTransmit:
;                  Transmit a packet. Block execution until it completes.
;                  This function attempts to transmit a packet. The address
;                  of the packet buffer should have previously been set with
;                  a call to RadioSetPtr.
;
;                  This routine gives the user very little control - probably
;                  less than most applications will require. This function is
;                  primarily intended for very simple applications that have
;                  no use for a time-out.
;
; 'C' Call:        RADIO_STATE RadioBlockingTransmit(BYTE retryCt,
;                                                    RADIO_LENGTH len);
;                  (A call to RadioSetPtr must have been made prior to the
;                  call to RadioBlockingTransmit.)
;
; Assembly Call:   A: retryCount
;                  X: length
;                  RadioPtr: Address of packet buffer
;                  
; Assembly Return: A: RADIO_STATE
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioBlockingTransmit::
 RadioBlockingTransmit::
        CALL    RadioStartTransmit
                   
.Wait:  CALL    RadioGetTransmitState
        MOV     [RadioTemp2], A
        TST     [RadioTemp2], TXC_IRQ | TXE_IRQ
        JZ      .Wait

        JMP     RadioEndTransmit        ; A = RADIO_STATE
.endsection

;--------------------------------------------------------------------------;
;                                                                          ;
;                 R E C E I V E                                            ;
;                                                                          ;
;--------------------------------------------------------------------------;
.section
;-----------------------------------------------------------------------------
;
; RadioStartReceive:
;                  Start the reception of a packet. The location and length of
;                  the packet buffer to receive the data into must have
;                  previously been set with a call to RadioSetPtr and
;                  RadioSetLength.
;
;                  After starting the reception of a packet with this call,
;                  the state of the receive operation should be checked by
;                  calling RadioGetReceiveState. When RadioGetReceiveState
;                  indicates that the transmission has completed a call
;                  should be made to RadioEndReceive.
;
;                  Receive is started by setting RX_GO" bit. All interesting
;                  interrupt enables are set and RadioGetReceiveState
;                  can be called in a polling loop in systems that do not use
;                  interrupts, or can be called directly in an interrupt
;                  handler.
;
;                  After calling RadioStartReceive NO CALLS can be made to the
;                  configuration access routines until receive operation is
;                  terminated with a call to RadioEndReceive or RadioAbort.
;                  Until a call is made to end the receive operation
;                  the only other calls supported are RadioGetReceiveState and
;                  RadioGetRssi.
;
; 'C' Call:        void RadioStartReceive(void);
;                  (A call to RadioSetPtr must have been made prior to the
;                  call to RadioStartReceive.)
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioStartReceive::
 RadioStartReceive::
        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], [RadioPtr+1] ; MSByte of pointer
  ENDIF
        MOV     [RadioWipPtr+0], [RadioPtr+0]
        MOV     [RadioWipLen],   [RadioLen]   ; sizeof(RadioPtr)
        MOV     [RadioState], RADIO_RX
        MOV     [RadioBytesRead], 0

        CALL    wakeupRadio

        MOV     A, RX_CTRL_ADR
        MOV     X, (RX_GO | RXC_IRQ | RXE_IRQ)
        CALL    RadioWrite

        MOV     A, RX_IRQ_STATUS_ADR
        JMP     RadioRead               // Clr SOP detect bit
.endsection
                    
.section
;-----------------------------------------------------------------------------
;
; RadioGetReceiveState:
;                  Returns the state of the current receive operation.
;                  This call should be made after RadioStartReceive()
;
;                  Although  bits in the state register in the hardware clear
;                  automatically, we make them sticky until RadioEndReceive.
;
; 'C' Call:        RADIO_STATE RadioGetReceiveState(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: State
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioGetReceiveState::
 RadioGetReceiveState::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        TST_IRQ_PIN
        JZ      RGRSDone                ; Nothing interesting, exit

RadioGetReceiveStateIsr:                ; Entry via RadioInterrupt() wrapper
        MOV     A, RX_IRQ_STATUS_ADR
        CALL    RadioReadStatusDebounced
        AND     A, (RXBERR_IRQ | RXE_IRQ | RXC_IRQ)
        OR      [RadioState], A         ; Make bits sticky

        TST     [RadioState], (RXBERR_IRQ | RXE_IRQ) ; If Error,
        JZ      RGRSDone                             ;  then set RXC/RXE
        OR      [RadioState], (RXC_IRQ | RXE_IRQ)    ;  because radio is done

RGRSDone:
        MOV     A, [RadioState]         ;  State calls and return them.
        RET                    
    
.endsection
    
.section
;-----------------------------------------------------------------------------
;
; RadioEndReceive: Complete a receive operation.
;
; 'C' Call:        RADIO_LENGTH RadioEndReceive(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: # of bytes copied to User's Buf
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioEndReceive::
 RadioEndReceive::
        MOV     A, RX_CFG_ADR           ; Read current VLD_EN setting
        CALL    RadioRead               ; (RadioRead also sets CUR_PP)
        MOV     [RadioTemp3], A         ;

        MOV     A, RX_COUNT_ADR         ; Total # bytes through RFIFO
        CALL    RadioRead               ; (RadioRead also sets CUR_PP)
        TST     [RadioTemp3], VLD_EN
        JZ      .S2                     ; Normal (VLD_EN == 0)
        ASL     A                       ; 2x, every data byte has a valid byte
.S2:    MOV     [RadioTemp3], A         ; # Bytes in RFIFO

        TST     [RadioState], RXE_IRQ   ;
        JZ      .S1                     ; No error, unload the final byte(s)
        MOV     A, LP_FIFO_SIZE         ; RXE, purge full RFIFO to recove
.S1:
        MOV     X, RX_BUFFER_ADR
        SWAP    A, X
        CALL    RadioFileReadWip        ; Final RFIFO unload

RadioRxCleanup:
        MOV     A, [RadioXactConfig]    ;
        OR      A, FRC_END_STATE        ; Force to User's desired end-state
        MOV     X, XACT_CFG_ADR         ;
        CALL    RadioWriteSwapped
.wait:  MOV     A, XACT_CFG_ADR         ; Wait for FRC_END_STATE bit in
        CALL    RadioRead               ;  XACT_CFG register to clear
        AND     A, FRC_END_STATE        ;  indicating the Force End is
        JNZ     .wait                   ;  complete.

        MOV     A, [RadioTemp3]         ; Byte count
        JMP     RadioEndTransmit
.endsection


.section
;-----------------------------------------------------------------------------
;
; RadioForceState:  Force radio to desired state NOW.
;                   Updates global RadioXacConfig shadow variable.
;                   Sometimes used to wake/sleep radio to measure voltage.
;
;  We must ensure SLEEP only transitions to IDLE (and recovers if problem).
;  RadioForceState(END_STATE_IDLE) MUST be called prior to TX_GO or RX_GO 
;   whenever radio may be in SLEEP.
;
; 'C' Call:        void RadioForceState(XACT_CONFIG endStateBitsOnly);
;
; Assembly Call:   A: RadioXactConfig END_STATE bits only, no ACK_EN, ACK_TO
;                  X: Undefined
;
; Assembly Return: A,X: Undefined
;-----------------------------------------------------------------------------
macro   IsRadioSleep:                   ; Ret ZERO if Radio is in SLEEP mode
        MOV     A, [RadioXactConfig]    ; Current state, at least it should be
        AND     A, END_STATE_MSK        ;
  if    END_STATE_SLEEP
        CMP     A, END_STATE_SLEEP      ; optional since END_STATE_SLEEP == 0
  endif
endm

_RadioForceState::
 RadioForceState::
        RAM_SETPAGE_CUR >RadioDriverRamPage

        ; Only invoke when radio is in SLEEP
        PUSH    A                       ; Save target state
        IsRadioSleep
        JNZ     .noCdt                  ; Current state IS NOT SLEEP
        MOV     A, END_STATE_IDLE       ; Get Radio IDLE before anything else
        CALL    forceState              ; Force SLEEP--->IDLE
.noCdt:
        POP     A                       ; Restore target state
        ; fallthru                      ; NON_SLEEP ---> User's Radio state

forceState:     
        CALL    frcState                ; Write State w/Force
        MOV     [RadioScratch0], 0      ; Delay 5 mS max timeout
wait5:  MOV     A, XACT_CFG_ADR         ;
        CALL    RadioRead               ;
        AND     A, FRC_END_STATE        ;
        JZ     return                   ; Wait for Force End State completion
        DEC     [RadioScratch0]
        JNZ     wait5                   ; No timeout yet
                ; timeout possible ONLY when going SLEEP to IDLE
        MOV     A, END_STATE_SLEEP
        CALL    frcState
        MOV     A, END_STATE_IDLE       ; Get Radio IDLE before anything else
        JMP     forceState

frcState:       
        AND     [RadioXactConfig],  ~END_STATE_MSK
        OR      A, [RadioXactConfig]    ; Merge w/existing ACK_EN, ACK_TO bits
        OR      A, FRC_END_STATE        ; Force the State
        JMP     RadioSetXactConfig

; ----------------------------------------------------------------------------
; wakeupRadio - If radio is in SLEEP, wake it up.  
;               Leave "unforced" END STATE in (RadioXactConfig) unchanged.
; ----------------------------------------------------------------------------
wakeupRadio:
        IsRadioSleep
        JNZ     return                  ; Radio already awake
        MOV     A, END_STATE_IDLE
        CALL    forceState              ; Radio SLEEP ---> IDLE

        MOV     A, [RadioXactConfig]    ; Restore unforced END state to SLEEP
        AND     A, ~END_STATE_MSK
  if    END_STATE_SLEEP         ; Only need OR if END_STATE_SLEEP != 0
        OR      A, END_STATE_SLEEP      ; optional since END_STATE_SLEEP == 0
  endif
        JMP     RadioSetXactConfig

return: RET
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioGetRssi:    Returns the receiver signal strength indicator.
;
; 'C' Call:        RADIO_RSSI RadioGetRssi(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: Rssi
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioGetRssi::
 RadioGetRssi::
        MOV     A, RSSI_ADR
        JMP     RadioRead
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioAbort:      Abort a receive operation.
;
; 'C' Call:        RADIO_LENGTH RadioAbort(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: RADIO_ABORT_SUCCESS or Length of valid Rx packet
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioAbort::
 RadioAbort::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        TST     [RadioState], RADIO_RX  ; This Abort handling is RX specific.
        JZ      .Abort

        ;  Don't issue Force End if receive has started.
        MOV     X, ABORT_EN             ; Try to avoid committing to a Receive
        MOV     A, RX_ABORT_ADR         ;  packet by invoking Digital Loopback
        CALL    RadioWrite              ;
        ; ------------------------------
        MOV     X, 64                   ; Delay to see if SOP arrives
.L1:    DEC     X       ; 4 cycles      ;
        JNZ     .L1     ; 5 cycles      ; (9 * 64) / 12 MHz = 48 uS
        ; ------------------------------
        MOV     A, RX_IRQ_STATUS_ADR    ; See if Rx packet has started
        CALL    RadioReadStatusDebounced;
        OR      [RadioState], A         ; Make bits sticky
        TST     [RadioState], SOFDET_IRQ
        JNZ     .RxOk                   ; Rx packet has started, finish it
        CALL    RadioRxCleanup          ; Rx packet blocked, issue FORCE_END

.Abort: MOV     A, RADIO_ABORT_SUCCESS
        JMP     .AbortExit

; ----------------------------------------------------------------------------
; Receiving a packet, allow to finish naturally.
; Probably has errors (RXE) because we activated Digital Loopback,
;  but if Loopback activated during AutoAck, we'll have a valid packet
;  (which was AutoAcked, so we'd better not discard it!).
; ----------------------------------------------------------------------------
.RxOk:  CALL    RadioGetReceiveState    ; Keep servicing receiver
        AND     A, RXE_IRQ | RXC_IRQ
        JZ      .RxOk
        CALL    RadioEndReceive         ; A = Rx result
        TST     [RadioState], RXE_IRQ
        JZ      .AbortExit              ; Rx good packet, A = packet length
        MOV     A, RADIO_ABORT_SUCCESS  ; Rx with error(s)

.AbortExit:
        PUSH    A
        MOV     X, 0
        MOV     A, RX_ABORT_ADR         ; Remove digital Loopback
        CALL    RadioWrite
        POP     A
        RET
.endsection

.section
;-----------------------------------------------------------------------------
;
; RadioPoll:       Same as RadioInterrupt() but pushes the Flag register on 
;                  the stack to setup for RETI.
;                  RadioPoll() can be called from a polling loop or an ISR
;                  that can not JMP - ie. an ISR written in 'C'.
;
;                  Unlike RadioInterrupt(), RadioPoll() may destroy registers.
;
; RadioInterrupt:  Manage the radio in an ISR or polling loop.
;
;                  For interrupt systems RadioInterrupt can be called from the
;                  GPIO interrupt. This function does nothing if the IRQ is
;                  not asserted thereby making it easy to share the IRQ with
;                  other GPIO interrupts.
;
;                  Using this function in an IRQ can eliminate latency caused
;                  by calling RadioGetTransmitState and/or 
;                  RadioGetReceveiveState in a polling loop. The state of the 
;                  radio is communicated to outside code through the global 
;                  variable RadioState.
;
;                  RadioInterrupt terminates with a RETI. It is intended to be
;                  the target of a JMP instruction from the interrupt vector
;                  directly in systems where the radio does not share the GPIO
;                  interrupt, or it can be JMP'd to at the end of a ISR that
;                  manages the other GPIO interrupt sources in a system.
;
;                  Because RadioInterrupt is intended to be JMP'd to from the
;                  interrupt vector it leaves the registers unaffected.
;
; 'C' Call:        void RadioInterrupt(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: Untouched
;                  X: Untouched
;-----------------------------------------------------------------------------
_RadioPoll::
 RadioPoll::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        PUSH_F_VIA_RAM                  ; PUSH F and disable GIE

; ----------------------------------------------------------------------------
; RadioInterrupt() - Jumped-to via an ISR (or simulated ISR)
;
; Assumes:      1. GIE is disabled
;               2. F register on top of stack (arrived via JMP from an ISR)
;               3. Direct Memory instructions access Page 0
;
; Large Memory Model: must preserve CUR_PP and possibly MVR_PP.
;                     may leave STK_PP = SYSTEM_STACK_PAGE on exit.
; ----------------------------------------------------------------------------
_RadioInterrupt::
 RadioInterrupt::
        TST_IRQ_PIN
        JZ      .exit0                  ; IRQ not asserted, just RETI

        PUSH    A                       ; Save Regs
        PUSH    X                       ;
        ISR_REG_PRESERVE (CUR_PP)       ; LMM saves CUR_PP reg

        TST     [RadioState], RADIO_TX
        JZ      .rxChk
.txChk:
        ISR_REG_PRESERVE (MVR_PP)       ; LMM saves MVR_PP
        CALL    RadioGetTransmitStateIsr  
        ISR_REG_RESTORE  (MVR_PP)       ; LMM restores MVR_PP
        JMP     .exit1                   

.rxChk: TST     [RadioState], RADIO_RX
        JZ      .exit1
        CALL    RadioGetReceiveStateIsr
        ; LMM Note:
        ;  RadioGetReceiveStateIsr() leaves STK_PP set to SYSTEM_STACK_PAGE.
        ;  Only a VERY unusual application would have a problem with this.
.exit1:
        ISR_REG_RESTORE (CUR_PP)        ; LMM restores CUR_PP
        POP     X                       ; Restore Regs
        POP     A                       ;
.exit0:
        POP_F_RET                       ; Same as RETI
.endsection


.section
;-----------------------------------------------------------------------------
;
; RadioGetReceiveStatus:
;                  Returns the receiver status register.
;
; 'C' Call:        RADIO_RX_STATUS RadioGetReceiveStatus(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: Status
;                  X: Undefined
;-----------------------------------------------------------------------------
_RadioGetReceiveStatus::
 RadioGetReceiveStatus::
        MOV     A, RX_STATUS_ADR
        JMP     RadioRead
.endsection

        ENABLE_CODE_COMPRESSION
; ############################################################################
; END OF lpNonStreaming.asm
; ############################################################################
