;--------------------------------------------------------------------------
;
;  Filename:    lpRadio.asm
;
;  Description: User functions to access the LP radio configuration. These
;               functions are the same for the both the streaming and
;               non-streaming versions of the driver.
;
;--------------------------------------------------------------------------
; WirelessUSB LP Radio Driver Version 1.2
;--------------------------------------------------------------------------
;
; Copyright 2005-2006, Cypress Semiconductor Corporation.
;
; This software is owned by Cypress Semiconductor Corporation (Cypress)
; and is protected by and subject to worldwide patent protection (United
; States and foreign), United States copyright laws and international
; treaty provisions. Cypress hereby grants to licensee a personal,
; non-exclusive, non-transferable license to copy, use, modify, create
; derivative works of, and compile the Cypress Source Code and derivative
; works for the sole purpose of creating custom software in support of
; licensee product to be used only in conjunction with a Cypress integrated
; circuit as specified in the applicable agreement. Any reproduction,
; modification, translation, compilation, or representation of this
; software except as specified above is prohibited without the express
; written permission of Cypress.
;
; Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
; WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
; Cypress reserves the right to make changes without further notice to the
; materials described herein. Cypress does not assume any liability arising
; out of the application or use of any product or circuit described herein.
; Cypress does not authorize its products for use as critical components in
; life-support systems where a malfunction or failure may reasonably be
; expected to result in significant injury to the user. The inclusion of
; Cypress' product in a life-support systems application implies that the
; manufacturer assumes all risk of such use and in doing so indemnifies
; Cypress against all charges.
;
; Use may be limited by and subject to the applicable Cypress software
; license agreement.
;
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------;
;                                                                          ;
;                      I N C L U D E   F I L E S                           ;
;                                                                          ;
;--------------------------------------------------------------------------;

                   INCLUDE "lpradio.inc"
                   INCLUDE "spim_radio.inc"
                   INCLUDE "psocgpioint.inc"
                   INCLUDE "lpregs.inc"
                   INCLUDE "lpIrqMacros.inc"
                   INCLUDE "memory.inc"

;--------------------------------------------------------------------------;
;
; PROVIDE_ALL_FUNCTIONS assembles all functions
;
; When set to ZERO, compiles only functions used by Keyboard/Mouse RDK
;
;--------------------------------------------------------------------------;
PROVIDE_ALL_FUNCTIONS:  EQU     1       ; KBM build sets to ZERO

;--------------------------------------------------------------------------;
;                                                                          ;
;                            V A R I A B L E S                             ;
;                                                                          ;
;--------------------------------------------------------------------------;

                   AREA    InterruptRAM(ram)    ; Variables on RAM Page 0

RadioDriverRamPage::                    ; used for page address only
_RadioXactConfig::
RadioXactConfig::  BLK     1            ; Current value of XACT_CFG_ADDR.

_RadioState::
 RadioState::      BLK     1            ; The current rx or tx state.

RadioTxCount::
RadioConstAddrLo:  BLK     1            ; Parameter to Radio TX / RX routines

;
; These variables are re-used to the point of confusion. They are commonly 
;  used for different functions, but occasionally used together as a buffer
;  to burst into the radio registers.  The RadioPacketLength field is used 
;  during async rx and tx operations and therefore it is NOT valid to call 
;  any functions that use it for any other purpose between a start/end
;  rx/tx pair.
;
RadioConstAddrHi:
RadioPacketLength::
RadioTemp1::       BLK     1            ; A work area used for short bursts.

RadioFileAddr:
RadioTxControl::
RadioTemp2::       BLK     1            ;  and other temp uses.

RadioCount:
RadioRetryCount::                                                      
RadioTemp3::       BLK     1

tmpPushF::         BLK     1            ; Used only during PUSH_F macro

;--------------------------------------------------------------------------;
;                                                                          ;
;                                 C O D E                                  ;
;                                                                          ;
;--------------------------------------------------------------------------;

        AREA    UserModules (ROM, REL)
        DISABLE_CODE_COMPRESSION        ; Code breaks if "subroutinized".

;--------------------------------------------------------------------------;
;                                                                          ;
;                      I N I T I A L I Z A T I O N                         ;
;                                                                          ;
;--------------------------------------------------------------------------;


.section
;--------------------------------------------------------------------------
;
; RadioInit:       Initialize the Radio.
;
; 'C' Call:        void RadioInit(XACT_CONFIG xactConfig, TX_CONFIG txConfig);
;
; Assembly Call:   A: xactConfig
;                  X: txConfig
;
; Assembly Return: A: Garbage
;                  X: Garbage
;
_RadioInit::
 RadioInit::
        RAM_SETPAGE_CUR >RadioDriverRamPage

        MOV     [RadioTemp1], A
        MOV     [RadioTemp2], X

        MOV     [RadioXactConfig], XACT_CFG_RST

        CALL    RadioReset

;
; We're going to want to force the RXF clock on in the streaming driver.
; Not necessary for nonstreaming driver but harmless.
;
        MOV     A, CLK_EN_ADR
        MOV     X, RXF
        CALL    RadioWrite

;
; Set up to use auto-cal for VCO. Register 0x34 is left at its default value
; to allow auto-cal to be used.
;
        MOV     A, AUTO_CAL_TIME_ADR    ; Extend autoCal time to max.
        MOV     X, AUTO_CAL_TIME_MAX
        CALL    RadioWrite

        MOV     A, AUTO_CAL_OFFSET_ADR  ; AutoCal offset to -4.
        MOV     X, AUTO_CAL_OFFSET_MINUS_4
        CALL    RadioWrite

        MOV     A, IO_CFG_ADR           ; Read IO_CFG_ADR, user may have
        CALL    RadioRead               ;  changed it in RadioReset
        OR      A, IRQ_POL              ; Set IRQ polarity to positive.
        MOV     X, IO_CFG_ADR           ;  (Some have asked that this be
        CALL    RadioWriteSwapped       ;  left at the default negative.)
;
; Set the XTAL Startup delay to 150uSec to handle warm restarts of the XTAL. 
;
        MOV     A, XTAL_CFG_ADR
        MOV     X, 0x08
        CALL    RadioWrite
;
; Enable HiLo for quick-turn-around. Use low side injection for receive - this
;  should result in non-inverted data, so no need to hit the invert bit. 
; Turn off AGC and force the LNA on.
; 
        MOV     A, RX_CFG_ADR
        MOV     X, ((RX_CFG_RST | FASTTURN_EN | LNA_EN) & ~(HI | RXOW_EN | AUTO_AGC_EN))
        CALL    RadioWrite
;
; Set the TX Offset to +1MHz.
;
; THIS MEANS THE ACTUAL TRANSMIT CARRIER WILL BE 1MHz ABOVE THE PLL
;  FREQUENCY PROGRAMMED IN THE CHANNEL OR A & N REGISTERS.
;  RadioSetFrequency COMPENSATES FOR THIS OFFSET.
;
        MOV     A, TX_OFFSET_LSB_ADR
        MOV     X, 055h
        CALL    RadioWrite
        MOV     A, TX_OFFSET_MSB_ADR
        MOV     X, 005h
        CALL    RadioWrite
;
; Set the radio transaction and TX configuration.
;                   
        MOV     A, [RadioTemp1]
        CALL    RadioSetXactConfig
        MOV     A, [RadioTemp2]
        CALL    RadioSetTxConfig

        MOV     [RadioState], RADIO_IDLE
        RET
.endsection

;--------------------------------------------------------------------------;
;                                                                          ;
;                        C O N F I G U R A T I O N                         ;
;                                                                          ;
;--------------------------------------------------------------------------;
.section
;-----------------------------------------------------------------------------
;
; RadioSetChannel:
;                  Set the channel (frequency + 2MHz).
;
; 'C' Call:        void RadioSetChannel(BYTE channel);
;
; Assembly Call:   A: frequency
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
; Note:            FALL THROUGH INTO RadioSetFrequency.
;
 RadioSetChannel::
_RadioSetChannel::
        ADD     A, 2
        ; fallthru to RadioSetFrequency()

.endsection
.section
;-----------------------------------------------------------------------------
;
; RadioSetFrequency:
;                  Set the frequency on which future communications will occur.
;
;                  THIS DRIVER SETS TX OFFESET AT 1 WHICH MEANS THAT THE TX
;                  CARRIER WILL BE 1MHz HIGHER THAN CHANNEL NUMBER WE PROGRAM.
;                  THIS FUNCTION COMPENSATES FOR THAT OFFSET BY DECREMENTING 
;                  THE CHANNEL NUMBER.
;
; 'C' Call:        void RadioSetFrequency(BYTE frequency);
;
; Assembly Call:   A: frequency
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
 RadioSetFrequency::
_RadioSetFrequency::
        CMP     A, 0
        JZ      .Chan0
        DEC     A
.Chan0: MOV     X, CHANNEL_ADR          ; Write into the channel register.
        JMP     RadioWriteSwapped

.endsection
.section
;-----------------------------------------------------------------------------
;
; RadioGetChannel:
;                  Get the channel.
;
; 'C' Call:        BYTE RadioGetChannel(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: channel
;                  X: Undefined
;
 RadioGetChannel::
_RadioGetChannel::
        CALL    RadioGetFrequency
        SUB     A, 2
        RET

.endsection
.section
;-----------------------------------------------------------------------------
;
; RadioGetFrequency:
;                  Returns the frequency in MHz above 2.4GHz. 
;                  Example: 0 means 2.400GHz, 83 means 2.483GHz.
;
; 'C' Call:        BYTE RadioGetFrequency(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: frequency
;                  X: Undefined
;
 RadioGetFrequency::
_RadioGetFrequency::
        MOV     A, CHANNEL_ADR          ;  into the channel register.
        CALL    RadioRead
        INC     A
        RET

.endsection
.section
;-----------------------------------------------------------------------------
;
; RadioSetTxConfig:Set the transmitter configuration.
;
; 'C' Call:        void RadioSetTxConfig(TX_CONFIG config);
;
; Assembly Call:   A: config
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetTxConfig::
 RadioSetTxConfig::
        MOV     X, TX_CFG_ADR
        JMP     RadioWriteSwapped
                    
.endsection
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetTxConfig:Return the transmitter configuration.
;
; 'C' Call:        TX_CONFIG RadioGetTxConfig(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: config
;                  X: Undefined
;
_RadioGetTxConfig::
 RadioGetTxConfig::
        MOV     A, TX_CFG_ADR
        JMP     RadioRead
                    
.endsection
ENDIF
.section
;-----------------------------------------------------------------------------
;
; RadioSetXactConfig:
;                  Set the transaction configuration.
;
; 'C' Call:        void RadioSetXactConfig(XACT_CONFIG config);
;
; Assembly Call:   A: config (user shouldn't set FRC_END_STATE bit)
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetXactConfig::
 RadioSetXactConfig::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioXactConfig], A    ;  use RadioSetState() goto a State.
        AND     [RadioXactConfig], ~FRC_END_STATE
        MOV     X, XACT_CFG_ADR
        JMP     RadioWriteSwapped
                   
                    
.endsection
.section
;-----------------------------------------------------------------------------
;
; RadioGetXactConfig:
;                  Return the transaction configuration.
;
; 'C' Call:        XACT_CONFIG RadioGetXactConfig(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: config
;                  X: Undefined
;
_RadioGetXactConfig::
 RadioGetXactConfig::
        MOV     A, XACT_CFG_ADR
        JMP     RadioRead
                    
.endsection
.section
;-----------------------------------------------------------------------------
;
; RadioSetFrameConfig:
;                  Set the framing configuration.
;
; 'C' Call:        void RadioSetFrameConfig(RADIO_FRAME_CONFIG config);
;
; Assembly Call:   A: config
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetFrameConfig::
 RadioSetFrameConfig::
        MOV     X, FRAMING_CFG_ADR
        JMP     RadioWriteSwapped
                    
.endsection
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetFrameConfig:
;                  Set the framing configuration.
;
; 'C' Call:        RADIO_FRAME_CONFIG RadioGetFrameConfig(void);
;
; Assembly Call:   A: Unused
;                  X: Unused
;
; Assembly Return: A: config
;                  X: Undefined
;
_RadioGetFrameConfig::
 RadioGetFrameConfig::
        MOV     A, FRAMING_CFG_ADR
        JMP     RadioRead
                    
.endsection
ENDIF
.section
;-----------------------------------------------------------------------------
;
; RadioSetThreshold32:
;                  Set the threshold for the 32 chip data modes.
;
; 'C' Call:        void RadioSetThreshold32(BYTE threshold);
;
; Assembly Call:   A: threshold
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetThreshold32::
 RadioSetThreshold32::
        MOV     X, DATA32_THOLD_ADR
        JMP     RadioWriteSwapped
                    
.endsection
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetThreshold32:
;                  Return the threshold for the 32 chip data modes.
;
; 'C' Call:        BYTE RadioGetThreshold32(void);
;
; Assembly Call:   A: unused
;                  X: unused
;
; Assembly Return: A: threshold
;                  X: Undefined
;
_RadioGetThreshold32::
 RadioGetThreshold32::
        MOV     A, DATA32_THOLD_ADR
        JMP     RadioRead
                    
.endsection
ENDIF
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioSetThreshold64:
;                  Set the threshold for the 64 chip data modes.
;
; 'C' Call:        void RadioSetThreshold64(BYTE threshold);
;
; Assembly Call:   A: threshold
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetThreshold64::
 RadioSetThreshold64::
        MOV     X, DATA64_THOLD_ADR
        JMP     RadioWriteSwapped
                    
.endsection
ENDIF
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetThreshold64:
;                  Return the threshold for the 64 chip data modes.
;
; 'C' Call:        BYTE RadioGetThreshold64(void);
;
; Assembly Call:   A: unused
;                  X: unused
;
; Assembly Return: A: threshold
;                  X: Undefined
;
_RadioGetThreshold64::
 RadioGetThreshold64::
        MOV     A, DATA64_THOLD_ADR
        JMP     RadioRead
                    
.endsection
ENDIF
.section
;-----------------------------------------------------------------------------
;
; RadioSetPreambleCount:
;                  Set the preamble repetition count.
;
; 'C' Call:        void RadioSetPreambleCount(BYTE count);
;
; Assembly Call:   A: preamble count
;                  X: Unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetPreambleCount::
 RadioSetPreambleCount::
        MOV     X, PREAMBLE_ADR         ; Write the preamble count...
        CALL    RadioWriteSwapped

        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSByte RadioWipPtr 
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; LSByte RadioWipPtr
                                        ; Preamble file register is 3 bytes
        MOV     A, PREAMBLE_ADR         ;  and must always be accessed 3 times
        MOV     X, 2                    ;  so do a throw away burst read of 2
        JMP     RadioFileReadWip        ;  to move it a total of 3 bytes.

.endsection
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetPreambleCount:
;                  Get the preamble repetition count.
;
; 'C' Call:        BYTE RadioGetPreambleCount(void);
;
; Assembly Call:   A: unused
;                  X: unused
;
; Assembly Return: A: preamble count
;                  X: Undefined
;
_RadioGetPreambleCount::
 RadioGetPreambleCount::
        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSB RadioWipPtr
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; SMM: 8-bit RadioWipPtr
                                        ; The preamble file register is 3 byte
        MOV     A, PREAMBLE_ADR         ;  and must always be accessed 3 times
        MOV     X, 3                    ;  so do a burst read of the whole 3
        CALL    RadioFileReadWip        ;  byte file register.
        MOV     A, [RadioTemp1]         ; The first byte is the count.
        RET
                    
.endsection
ENDIF
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioSetPreamblePattern:
;                  Set the preamble pattern.
;
; 'C' Call:        void RadioSetPreamblePattern(WORD pattern);
;
; Assembly Call:   A: pattern low order
;                  X: pattern high order
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetPreamblePattern::
 RadioSetPreamblePattern::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioTemp1], A         ; Save the pattern.
        MOV     [RadioTemp2], X
        MOV     A, PREAMBLE_ADR         ; Gotta access the whole 3 byte file -
        CALL    RadioRead               ;  do throw away read of preamblecount

  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSB RadioWipPtr
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; SMM: 8-bit RadioWipPtr
        MOV     A, PREAMBLE_ADR         ; Write the pattern as a 2 byte
        MOV     X, 2                    ;  file register burst.
        JMP     RadioFileWriteWip

.endsection
ENDIF
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetPreamblePattern:
;                  Get the preamble pattern.
;
; 'C' Call:        WORD RadioGetPreamblePattern(void);
;
; Assembly Call:   A: unused
;                  X: unused
;
; Assembly Return: A: preamble count
;                  X: Undefined
;
_RadioGetPreamblePattern::
 RadioGetPreamblePattern::
        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSB RadioWipPtr
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; SMM: 8-bit RadioWipPtr
                                        ; Preamble file register is 3 bytes
        MOV     A, PREAMBLE_ADR         ;  and must always be accessed 3 times
        MOV     X, 3                    ;  so do a burst read of the whole 3
        CALL    RadioFileReadWip        ;  byte file register.
        MOV     A, [RadioTemp2]         ; The first byte is the count and the
        MOV     X, [RadioTemp3]         ;  pattern is the 2nd and 3rd bytes.
        RET
                    
.endsection
ENDIF
.section
;-----------------------------------------------------------------------------
;
; RadioSetCrcSeed: Sets CRC seed value used for both Tx and Rx
;
; 'C' Call:        void RadioSetCrcSeed(WORD crcSeed);
;
; Assembly Call:   A: crcSeed low order
;                  X: crcSeed high order
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
 RadioSetCrcSeed::
_RadioSetCrcSeed::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioTemp1], A         ; Save pattern in local work buffer
        MOV     [RadioTemp2], X
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSB RadioWipPtr
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; SMM: 8-bit RadioWipPtr
        MOV     A, CRC_SEED_LSB_ADR     ;  Write the pattern as a 2 byte file
        MOV     X, 2                    ;  register burst.
        JMP     RadioBurstWriteWip

.endsection
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetCrcSeed: Return CRC seed value used for both Tx and Rx
;
; 'C' Call:        WORD RadioGetCrcSeed(void);
;
; Assembly Call:   A: unused
;                  X: unused
;
; Assembly Return: A: crcSeed low order
;                  X: crcSeed high order
;
 RadioGetCrcSeed::
_RadioGetCrcSeed::
        RAM_SETPAGE_CUR >RadioDriverRamPage
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     [RadioWipPtr+1], >RadioTemp1 ; LMM: MSB RadioWipPtr
  ENDIF
        MOV     [RadioWipPtr+0], <RadioTemp1 ; SMM: 8-bit RadioWipPtr
                                        ; The 3-byte Preamble file register
        MOV     A, CRC_SEED_LSB_ADR     ;  must always be accessed 3 times
        MOV     X, 2                    ;  so do a burst read of whole 3
        CALL    RadioBurstReadWip       ;  byte file register.
        MOV     A, [RadioTemp1]         ; The first byte is the count and the
        MOV     X, [RadioTemp2]         ;  pattern is the 2nd and 3rd bytes.
        RET
                    
.endsection
ENDIF
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioGetFuses:   Get the fuse values from the radio.
;
;                  Copy 6 bytes of Fuse data to buf specified by RadioSetPtr.
;
; 'C' Call:        void RadioGetFuses(void);
;
; Assembly Call:   A: unused
;                  X: unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
 RadioGetFuses::
_RadioGetFuses::
        MOV     A, MFG_ID_ADR           ; Turn on fuse read bias.
        MOV     X, 0FFh
        CALL    RadioWrite

        MOV     A, MFG_ID_ADR           ; Read the fuses.
        MOV     X, 6
        CALL    RadioFileRead

;       RAM_SETPAGE_CUR >RadioDriverRamPage    already done by RadioWrite()
  IF (SYSTEM_LARGE_MEMORY_MODEL)
        MOV     A, [RadioPtr+1]         ; MSB RadioPtr (page number)
        RAM_SETPAGE_IDX A               ; IDX_PP = MSB
        RAM_X_POINTS_TO_INDEXPAGE       ; IDX instructions use IDX_PP reg
  ENDIF
        MOV     X, [RadioPtr+0]         ; LSB of pointer
        MOV     A, 5
.Loop:  XOR     [X+0], 0xFF
        INC     X
        DEC     A
        JNC     .Loop

        RAM_RESTORE_NATIVE_PAGING

        MOV     A, MFG_ID_ADR           ; Turn off fuse read bias.
        MOV     X, 0
        JMP     RadioWrite
.endsection
ENDIF
.section
;-----------------------------------------------------------------------------
;
; RadioSetConstSopPnCode:  Set SOP PN Code.
;
; 'C' Call:        void RadioSetConstSopPnCode(const BYTE *patternAddr);
;
; Assembly Call:   A: Code address high
;                  X: Code address low
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetConstSopPnCode::
 RadioSetConstSopPnCode::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioFileAddr], SOP_CODE_ADR
RadioSetConstCode2:
        MOV     [RadioConstAddrHi], A
        MOV     [RadioConstAddrLo], X
RadioSetConstCode:
        MOV     [RadioCount], 8
.loop:  MOV     A, [RadioConstAddrHi]
        MOV     X, [RadioConstAddrLo]
        ROMX
        INC     [RadioConstAddrLo]
        JNZ     .S1
        INC     [RadioConstAddrHi]
.S1:    MOV     X, [RadioFileAddr]
        CALL    RadioWriteSwapped
        DEC     [RadioCount]
        JNZ     .loop
        RET             

.endsection
IF PROVIDE_ALL_FUNCTIONS
.section
;-----------------------------------------------------------------------------
;
; RadioSetConstDataPnCode: Set DATA PN Code.
;
; 'C' Call:        void RadioSetConstDataPnCode(const BYTE *patternAddr);
;
; Assembly Call:   A: Code address high
;                  X: Code address low
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetConstDataPnCode::
 RadioSetConstDataPnCode::
        RAM_SETPAGE_CUR >RadioDriverRamPage
        MOV     [RadioFileAddr], DATA_CODE_ADR
        CALL    RadioSetConstCode2
        JMP     RadioSetConstCode

.endsection
ENDIF
.section
;-----------------------------------------------------------------------------
;
; RadioSetSopPnCode: Set the Start Of Packet PN Code.
;
; 'C' Call:        void RadioSetSopPnCode(BYTE patternNum);
;
; Assembly Call:   A: Code number
;                  X: unused
;
; Assembly Return: A: Undefined
;                  X: Undefined
;
_RadioSetSopPnCode::
 RadioSetSopPnCode::
        ASL     A                           ; PN Codes are 8 bytes.
        ASL     A
        ASL     A
        MOV     X, >RadioPnCodeTable
        ADD     A, <RadioPnCodeTable
        JNC     .S1
        INC     X
.S1:    SWAP    A, X
        JMP     _RadioSetConstSopPnCode

;--------------------------------------------------------------------------;
;                                                                          ;
;                        P N   C o d e   T a b l e s                       ;
;                                                                          ;
;--------------------------------------------------------------------------;

;-----------------------------------------------------------------------------
;
; These are the 20 Multiplicative codes documented in RWW-87. The first 10 are
;  the set recommended as the best at the conclusion of that memo. 
;
; Bit ordering for these codes:
;  The LSB of the first byte is the first bit transceived.
;  The MSB of the first byte is the eighth bit transceived.
;  The MSB of the eighth byte is the last bit transceived.
;
RadioPnCodeTable:  DB   3Ch, 37h, CCh, 91h, E2h, F8h, CCh, 91h ; PN Code 0
                   DB   9Bh, C5h, A1h, 0Fh, ADh, 39h, A2h, 0Fh ; PN Code 1 
                   DB   EFh, 64h, B0h, 2Ah, D2h, 8Fh, B1h, 2Ah ; PN Code 2 
                   DB   66h, CDh, 7Ch, 50h, DDh, 26h, 7Ch, 50h ; PN Code 3 
                   DB   5Ch, E1h, F6h, 44h, ADh, 16h, F6h, 44h ; PN Code 4 
                   DB   5Ah, CCh, AEh, 46h, B6h, 31h, AEh, 46h ; PN Code 5 
                   DB   A1h, 78h, DCh, 3Ch, 9Eh, 82h, DCh, 3Ch ; PN Code 6 
                   DB   B9h, 8Eh, 19h, 74h, 6Fh, 65h, 18h, 74h ; PN Code 7 
                   DB   DFh, B1h, C0h, 49h, 62h, DFh, C1h, 49h ; PN Code 8 
                   DB   97h, E5h, 14h, 72h, 7Fh, 1Ah, 14h, 72h ; PN Code 9 
;
; Most systems do not need 20 codes and they take up much space, so we've
;  commented out most of the codes. Uncomment them if you need more.
;
;                   DB   82h, C7h, 90h, 36h, 21h, O3h, FFh, 17h ; PN Code 10 
;                   DB   E2h, F8h, CCh, 91h, 3Ch, 37h, CCh, 91h ; PN Code 11 
;                   DB   ADh, 39h, A2h, 0Fh, 9Bh, C5h, A1h, 0Fh ; PN Code 12 
;                   DB   D2h, 8Fh, B1h, 2Ah, EFh, 64h, B0h, 2Ah ; PN Code 13 
;                   DB   DDh, 26h, 7Ch, 50h, 66h, CDh, 7Ch, 50h ; PN Code 14 
;                   DB   ADh, 16h, F6h, 44h, 5Ch, E1h, F6h, 44h ; PN Code 15 
;                   DB   B6h, 31h, AEh, 46h, 5Ah, CCh, AEh, 46h ; PN Code 16 
;                   DB   9Eh, 82h, DCh, 3Ch, A1h, 78h, DCh, 3Ch ; PN Code 17 
;                   DB   6Fh, 65h, 18h, 74h, B9h, 8Eh, 19h, 74h ; PN Code 18 
;                   DB   62h, DFh, C1h, 49h, DFh, B1h, C0h, 49h ; PN Code 19 

.endsection

        ENABLE_CODE_COMPRESSION
; ############################################################################
; END OF lpRadio.asm
; ############################################################################

